@@ -1,4 +1,4 @@
-## Copyright (C) 2012 Nir Krakauer
+## Copyright (C) 2012-2018 Nir Krakauer
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -28,7 +28,7 @@
 ##
 ## @var{x} and @var{w} should be @var{n} by 1 in size; @var{y} should be @var{n} by @var{m}; @var{xi} should be @var{k} by 1; the values in @var{x} should be distinct and in ascending order; the values in @var{w} should be nonzero.
 ##
-## Returns the selected @var{p}, the estimated data scatter (variance from the smooth trend) @var{sigma2}, and the estimated uncertainty (SD) of the smoothing spline fit at each @var{x} value, @var{unc_y}.
+## Returns the smoothing spline @var{pp} or its values @var{yi} at the desired @var{xi}; the selected @var{p}; the estimated data scatter (variance from the smooth trend) @var{sigma2}; the estimated uncertainty (SD) of the smoothing spline fit at each @var{x} value, @var{unc_y}; and the estimated number of degrees of freedom @var{df} (out of @var{n}) used in the fit.
 ##
 ## For small @var{n}, the optimization uses singular value decomposition of an @var{n} by @var{n} matrix  in order to quickly compute the residual size and model degrees of freedom for many @var{p} values for the optimization (Craven and Wahba 1979). For large @var{n} (currently >300), an asymptotically more computation and storage efficient method that takes advantage of the sparsity of the problem's coefficient matrices is used (Hutchinson and de Hoog 1985).
 ##
@@ -46,12 +46,14 @@
 ##
 ## Grace Wahba (1983), Bayesian ``confidence intervals'' for the cross-validated smoothing spline, J Royal Statistical Society, 45B:133-150
 ##
+## Herman J. Woltring (1986), A Fortran package for generalized, cross-validatory spline smoothing and differentiation, Advances in Engineering Software, 8(2):104–113
+##
 ## @end deftypefn
 ## @seealso{csaps, spline, csapi, ppval, dedup, bin_values, gcvspl}
 
 ## Author: Nir Krakauer <nkrakauer@ccny.cuny.edu>
 
-function [ret,p,sigma2,unc_y]=csaps_sel(x,y,xi,w,crit)
+function [ret,p,sigma2,unc_y,df]=csaps_sel(x,y,xi,w,crit)
 
   if (nargin < 5)
     crit = [];
@@ -124,27 +126,37 @@ function [ret,p,sigma2,unc_y]=csaps_sel(x,y,xi,w,crit)
 
   if ~exist("p", "var")
     ##choose p by minimizing the penalty function
-
-    p = fminbnd(penalty_function, 0, 1);
-
+    ##minimize over pt = log(-log(p)+1) to find p with good fractional accuracy if the optimum is very small, without making TolX very small
+    tol_pt = 1E-6;
+    pt_max = log(-log(realmin)+1) + 2*tol_pt;
+    pt = fminbnd(@(pt) penalty_function(exp(1-exp(pt))), 0, pt_max, optimset ("TolX", tol_pt));
+    p = exp(1-exp(pt));
   endif
 
   ## estimate the trend uncertainty
-  if chol_method
-    [MSR, Ht] = penalty_terms_chol(p, QT, R, y, w, n);
-    Hd = influence_matrix_diag_chol(p, QT, R, y, w, n);
-  else
-    H = influence_matrix(p, U, D, n, w);
-    [MSR, Ht] = penalty_terms(H, y, w);
-    Hd = diag(H);
-  end
-
-  sigma2 = mean(MSR(:)) * (n / (n-Ht)); #estimated data error variance (wahba83)
-  unc_y = sqrt(sigma2 * Hd ./ w); #uncertainty (SD) of fitted curve at each input x-value (hutchinson86)
-
-  ## construct the fitted smoothing spline 
-  [ret,p]=csaps(x,y,p,xi,w);
-
+  if isargout (3) || isargout (4) || isargout (5)
+    if chol_method
+      [MSR, df] = penalty_terms_chol(p, QT, R, y, w, n);
+    else
+      H = influence_matrix(p, U, D, n, w);
+      [MSR, df] = penalty_terms(H, y, w);
+    end
+    sigma2 = mean(MSR(:)) * (n / (n-df)); #estimated data error variance (wahba83)
+    if isargout (4)
+      if chol_method
+        Hd = influence_matrix_diag_chol(p, QT, R, y, w, n);
+      else
+        Hd = diag(H);
+      endif
+      unc_y = sqrt(sigma2 * Hd ./ w); #uncertainty (SD) of fitted curve at each input x-value (hutchinson86)
+    endif    
+  endif
+  
+  ## construct the fitted smoothing spline
+  if isargout (1)
+    ret = csaps (x,y,p,xi,w);
+  endif
+  
 endfunction
 
 
@@ -256,10 +268,24 @@ endfunction
 %!assert (ret, (csaps_sel(x,[y 2*y],x))'(:, 1), 1E-4);
 %!assert (unc_y, zeros(size(unc_y)), 1E-5);
 
+%!demo
+%! ni = 400; #number of evaluation points
+%! n = 40; #number of given sample points
+%! f = @(x) sin (2*pi*x); #function generating the synthetic data
+%! x = sort (rand (n, 1)); #sampled points
+%! y = f(x) + 0.1*randn (n, 1);
+%! xi = linspace (0, 1, ni); #evaluation points
+%! yi = csaps_sel (x,y,xi,[],'aicc');
+%! plot (x, y, 's', xi, yi)
+%! grid on
+%! legend ('Given data', 'Spline fit')
+%! title ('Spline smoothing with synthetic data sampled from sine wave with noise')
+
 %{
 #experiments comparing different selection criteria for recovering a function sampled with standard normal noise -- aicc was consistently better than aic, but otherwise which method does best is problem-specific
-m = 1; #number of replicates available
-ni = 200; #number of evaluation points
+tic
+m = 500; #number of replicates available
+ni = 400; #number of evaluation points
 ns = [5 10 20 40]; #number of given sample points
 nk = 100; #number of trials to average over
 f = @(x) sin(2*pi*x); #function generating the synthetic data
@@ -282,5 +308,6 @@ for i = 1:numel(ns)
   endfor
 endfor
 msem = mean(mse, 3);
+toc
 %}
 
