@@ -39,6 +39,14 @@ function [x, y, w] = dedup(x, y, w=ones(size(x)), tol=0, nan_remove=true)
 
 warning ("off", "Octave:broadcast", "local");
 
+if isempty(w)
+  w = ones(size(x));
+endif
+
+if isempty(tol)
+  tol = 0;
+endif
+
 if nan_remove
   #remove any rows with missing entries
   notnans = !any (isnan ([x y w]) , 2);
